@@ -203,7 +203,8 @@ int parse_lparcfg()
 int parse_proc_ints()
 {
 	FILE *f;
-	char line[512];
+	char *line, *p;
+	size_t n = 0;
 	char *value;
 	struct sysentry *se;
 	long long int phint = 0;
@@ -214,18 +215,25 @@ int parse_proc_ints()
 		return -1;
 	}
 
-	while (fgets(line, 512, f) != NULL) {
+	while (getline(&line, &n, f) != -1) {
+		p = line;
+		while (*p == ' ')
+			p++;
+
 		/* we just need the SPU line */
-		if (line[0] != 'S' || line[1] != 'P' || line[2] != 'U')
+		if (p[0] != 'S' || p[1] != 'P' || p[2] != 'U')
 			continue;
 
-		for (value = &line[5]; value[2] != 'S'; value += 11) {
+		for (value = &p[5]; value[2] != 'S'; value += 11) {
 			int v;
 			v = atoi(value);
 			phint += v;
 		}
+
+		break;
 	}
 
+	free(line);
 	fclose(f);
 
 	se = get_sysentry("phint");
@@ -264,11 +272,11 @@ int parse_proc_stat()
 	statvals[0] = 0;
 	value = line;
 	for (i = 1; i <= (entries - 1); i++) {
-		int v;
+		long long v;
 		value = strchr(value, ' ') + 1;
 		if (i == 1)
 			value++;
-		v = atoi(value);
+		v = atoll(value);
 		statvals[i] = v;
 		statvals[0] += v;
 	}
@@ -465,13 +473,13 @@ long long get_cpu_time_diff()
 
 void get_cpu_stat(struct sysentry *se, char *buf)
 {
-	float total, percent;
-	float old_val, new_val;
+	float percent;
+	long long total, old_val, new_val;
 
 	total = get_cpu_time_diff();
-	new_val = atoi(se->value);
-	old_val = atoi(se->old_value);
-	percent = (float)((new_val - old_val)/total) * 100;
+	new_val = atoll(se->value);
+	old_val = atoll(se->old_value);
+	percent = ((new_val - old_val)/(long double)total) * 100;
 	sprintf(buf, "%.2f", percent);
 }
 
@@ -569,10 +577,15 @@ void print_default_output(int interval, int count)
 	} while (--count > 0);
 }
 
+static struct option long_opts[] = {
+	{"version",	no_argument,	NULL,	'V'},
+	{0, 0, 0, 0},
+};
+
 int main(int argc, char *argv[])
 {
 	int interval = 0, count = 0;
-	int c;
+	int c, opt_index = 0;
 	int i_option = 0;
 
 	if (get_platform() != PLATFORM_PSERIES_LPAR) {
@@ -581,11 +594,15 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	while ((c = getopt(argc, argv, "i")) != -1) {
+	while ((c = getopt_long(argc, argv, "iV",
+				long_opts, &opt_index)) != -1) {
 		switch(c) {
 			case 'i':
 				i_option = 1;
 				break;
+			case 'V':
+				printf("lparstat - %s\n", VERSION);
+				return 0;
 			case '?':
 			default:
 				break;
